 /*                 xising
 
   Xwindow program to simulate the Ising model using microcanonical 
   algorithms.  Many "demons" circulate around the lattice
   trying to flip spins. 

   The latest version of this program can be found at
    http://penguin.phy.bnl.gov/www/xtoys/xtoys.html

  Michael Creutz   
  creutz@wind.phy.bnl.gov 

  this is version 2.5, 21 March 1995
  version 1.0 was for SPARC's only
  version 2.0 for generic X systems
  version 2.1 colored the buttons and thermometer fluid
  version 2.2 fixes bit ordering problems for DEC alpha's
  version 2.3 allows lattice resizing with the window
  version 2.4 fixes a deallocation problem
  version 2.5 cosmetic changes 

compiling:  cc -O -o xising xising.c -lm -lX11

If you find some machine supporting X that it does not work on, 
please let me know.

I have not included a canonical algorithm because 1) for the
local algorithm it is noticeably slower, and 2) I don't believe
that one would visually be able to distinguish any other difference 
beyond a non-fluctuating thermometer. */

# include <X11/Xlib.h>
# include <X11/Xutil.h>
# include <X11/Xos.h>
# include <X11/Xatom.h> 
# include <stdio.h>
# include <stdlib.h>
# include <math.h>
# include <limits.h>

 /* initial lattice dimensions:
        ncols will be truncated to a multiple of WORDSIZE 
        nrows will be changed if a multiple of 29            */
int nrows=256,ncols=256;
long **field=NULL;      /* to store the system */

 /* some convenient definitions */
# define WORDSIZE  (8*sizeof(long)) 
# define LEFTBIT  (~LONG_MAX)
/* (((long) 1)<<(WORDSIZE-1)) (changed 5/26/95 to remove warnings on sun) */
# define nextrow (row+1-nrows*(row>=nrows-1))
# define nextcol (col+nshift*(1-hwords*(col==topcol)))
# define previousrow (row-1+nrows*(row<1))
# define previouscol (col-nshift*(1-hwords*(col==bottomcol)))

long mrand48(),lrand48(); /* generates a random word, lrand is non-negative */
double drand48(); /* generates a random double */
char stringbuffer[100];
static char *progname;
char bits[256]; /* for counting bits */
int hwords,volume;

/* bitcount counts the set bits in a word; call setup() before using */
# define bitcount8(i)  (bits[(i)&255])
# define bitcount16(i) (bitcount8(i)+bitcount8((i)>>8))
/* check if long int is 32 bits */
#if ((0x7FFFFFFF)==(LONG_MAX))  /* 32 bits */
# define bitcount(i)   (bitcount16(i)+bitcount16((i)>>16))
# define RND mrand48()
#else  /* assume 64 bits */
# define bitcount32(i)   (bitcount16(i)+bitcount16((i)>>16))
# define bitcount(i)   (bitcount32(i)+bitcount32((i)>>32))
# define RND (mrand48()^(mrand48()<<32))
#endif

/* things to monitor temperature, etc. */
double beta; /* the inverse temperature */
long energycount=0, volumecount=0;
int heater=0, cooler=0;
int boundary=0, algorithm=0, paused=0, iteration=0;
  /* variables to allow for peculiar bit orderings on some machines */
int nshift, topcol, bottomcol;

/* for local algorithm; use two demon bits */
long demon0=0, demon1=0, checkermask;
long *work0=NULL, *work1=NULL;

/* stuff for the cluster algorithm; use three demon bits */
# define DEMONBITS 3
int ndemons;
long **cluster=NULL;    /* to store cluster shape */
long **sadx=NULL,**sady=NULL; /* labels sad demons */
int **unchecked=NULL; /* to reduce redundant calculations */
int boxtop,boxbottom,boxleft,boxright; /* will enclose cluster */
long **demon=NULL; /* for demons in cluster algorithm */
long **newdemon=NULL;
long demonindex=0; /* for shuffling demons */
long clustergrowth=0;
int direction=1; /* for vertical sweeps through cluster */

/* various window stuff */
Display *display;
int screen;
Window window,quitbutton,pausebutton,algobutton,
    heatwindow,boundwindow,makebutton();
GC gc,gcpen,gcclear;
XImage *spinimage=NULL,*clusterimage=NULL;
XFontStruct *font=NULL;
int font_height,font_width;
unsigned int windowwidth, windowheight;
XSizeHints size_hints;
int depth,darkcolor,lightcolor,black,white;
long event_mask;
void drawbutton(),openwindow(),setup(),makebuttons(),growcluster(),
     startcluster(),localupdate(),repaint(),setalg(),myalgo(),
     setheater(),setboundary(),fixthermometer(),check(),
     leftshift(),rightshift(),allocarray();
char * boundtext[4]={"periodic","antiperiodic","black edges","white edges"};
char * heatertext[3]={"conserve E","heat","cool"};
char * algotext[2]={"local","cluster"};

/* dimensions for control boxes */
#define HEATERHEIGHT 54
#define HEATERWIDTH 96
#define BOUNDHEIGHT 72
#define BOUNDWIDTH  110
#define BUTTONWIDTH 48
#define BUTTONHEIGHT 18

void main(argc,argv)
int argc;
char **argv;
{unsigned int width, height;
 XEvent report;
 progname=argv[0];
 openwindow(argc,argv);
 setup();
 makebuttons();
   /* loop forever, looking for events */
 while(1)
  {if (clustergrowth)  /* finish cluster regardless of algorithm */
     while (clustergrowth) 
      {growcluster(); 
      }
   else if (0==paused)
    {if (1==algorithm)
       {startcluster();
       }
     else localupdate();
    }
   if (paused|XPending(display))
    {XNextEvent(display,&report); 
     switch (report.type)
      {case Expose:
        if ((report.xexpose.window)!=window) break; /* cuts down flashing,
           but you might remove this line if things aren't being redrawn */
        if (report.xexpose.count!=0) break; /* more in queue, wait for them */
        repaint();  
        break;
       case ConfigureNotify:
        width=report.xconfigure.width;
        height=report.xconfigure.height;
        if ((width<size_hints.min_width)||(height<size_hints.min_height))
            {fprintf(stderr,"%s: window too small to proceed.\n",progname);
             XUnloadFont(display,font->fid);
             XFreeGC(display,gc); 
             XFreeGC(display,gcpen); 
             XFreeGC(display,gcclear); 
             XCloseDisplay(display);
             exit(1);
            } 
        if ((width!=windowwidth)||(height!=windowheight))
         {windowwidth=width;
          windowheight=height;
          ncols=width-75;
          nrows=(height-164)/2;
          makebuttons();
         }
        break; 
       case ButtonPress:
        if (report.xbutton.window==quitbutton)
            {XUnloadFont(display,font->fid);
             XFreeGC(display,gc); 
             XFreeGC(display,gcpen); 
             XFreeGC(display,gcclear); 
             XCloseDisplay(display);
             exit(1);
            } 
        else if (report.xbutton.window==pausebutton)
            {paused=1-paused;
             drawbutton(pausebutton,0,0,BUTTONWIDTH,BUTTONHEIGHT,
                 "pause",1-2*paused);
            }
        else if (report.xbutton.window==algobutton)
            {algorithm=1-algorithm;
             setalg();
             myalgo();
            }
        else if (report.xbutton.window==heatwindow)
            {setheater((report.xbutton.y*3)/HEATERHEIGHT);
            }
        else if (report.xbutton.window==boundwindow)
            {setboundary((report.xbutton.y*4)/BOUNDHEIGHT);
            }
        break;
       default:
        break;
      }
    }
  }
}

void startcluster()
{long xedge,yedge;
 int row,col,bit;
 long currentword;
 long index0,index1;
 /* flip previous cluster and adjust demons on boundary */
    for (row=0;row<nrows;row++)
      {for (col=0;col<hwords;col++)
         {index0=(demonindex+2*(col+row*hwords));
          index1=(index0+1);
          while (index0>=ndemons) index0-=ndemons; /* faster than using % */
          while (index1>=ndemons) index1-=ndemons;
          if (0==unchecked[row][col]) /* don't bother if unchecked */
           {unchecked[row][col]=1;    /* reset unchecked */
            currentword=cluster[row][col];
             /* find cluster edges */
            xedge=currentword^((currentword<<1)
              |(1&(cluster[row][nextcol]>>(WORDSIZE-1))));
            yedge=currentword^cluster[nextrow][col];
             /* update demons and field */
            for (bit=0;bit<DEMONBITS;bit++)
             {demon[bit][index0]
               =(newdemon[bit][index0]&xedge)|((~xedge)&demon[bit][index0]);
              demon[bit][index1]
               =(newdemon[bit][index1]&yedge)|((~yedge)&demon[bit][index1]);
             }
            field[row][col]^=currentword;
             /* do any cooling or heating required */
            if (drand48()<0.05)
             {if (heater) 
               {demon[0][index0]^=RND;
                demon[0][index1]^=RND;
               }
              else if (cooler)
               {demon[0][index0]&=RND;
                demon[0][index1]&=RND;
               }
              else /* rotate the demon's first bit to help scramble */
               {demon[0][index0]=(demon[0][index0]<<1)
                   |(1&(demon[0][index0]>>(WORDSIZE-1)));
                demon[0][index1]=(demon[0][index1]<<1)
                   |(1&(demon[0][index1]>>(WORDSIZE-1)));
              }
             }
           } /* end of if checked */
            /* accumulate counts */
          currentword=demon[0][index0];
          energycount+=bitcount(currentword);
          currentword=demon[0][index1];
          energycount+=bitcount(currentword);
          volumecount+=WORDSIZE; 
         } /* end of col loop */
      } /* end of row loop */
/* clear previous cluster */
    for (col=0;col<hwords;col++)
      for (row=0;row<nrows;row++)
         cluster[row][col]=0;
/* randomize starting demon location */
  demonindex=lrand48()%ndemons;
/* pick site to seed cluster */
  row=lrand48()%nrows;
  col=lrand48()%hwords;
  clustergrowth=cluster[row][col]=1<<(lrand48()%WORDSIZE);
/* set up bounding box for cluster */
  boxleft=col+hwords-(1&(clustergrowth>>(WORDSIZE-1)));
  boxright=col+hwords+1+(clustergrowth&1);
  boxbottom=previousrow+nrows;
  boxtop=boxbottom+3;
  /* monitor temperature */
if (volumecount>=volume) /* update beta and thermometer */
 {beta=.5*log((double) (-1.+volumecount/(0.5*energycount))); 
  fixthermometer(); 
  energycount=0;
  volumecount=0;
 } /* end of temperature monitoring */
  /* copy lattice to window */
 XPutImage(display,window,gc,spinimage,0,0,64,124,ncols,nrows); 
 return;
} /* end of startcluster */

void growcluster()
/* the cluster growing routine */
{long newsites,temp;
 int row,col,rowloop,colloop,newboxbottom,newboxtop,newboxleft,newboxright;
 long currentword,temporary;
   newboxbottom=nrows;
   newboxtop=0;
   newboxleft=hwords;
   newboxright=0;
   clustergrowth=0;
   direction=(-direction); /* sweep rows in alternate directions */
   for (colloop=boxleft;colloop<boxright;colloop++)
      {col=colloop;
       while (col>=hwords) col-=hwords;
       for (rowloop=boxbottom;rowloop<boxtop;rowloop++)
         {if (direction<0)
            row=(boxbottom+boxtop-rowloop-1);
          else 
            row=rowloop;
          while (row>=nrows) row-=nrows;
           /* make sure sadx and sady for current word are calculated */
          if (unchecked[row][col])
           check(row,col);
          if (unchecked[previousrow][col])
           check(previousrow,col);
          if (unchecked[row][previouscol])
           check(row,previouscol);
          newsites=currentword=cluster[row][col];
          temporary=sadx[row][col];
          newsites|=
            (sady[row][col]&cluster[nextrow][col]); /* up */
          newsites|=
            (sady[previousrow][col]&cluster[previousrow][col]); /* down */
          newsites|=
           (temporary&1&(cluster[row][nextcol]>>(WORDSIZE-1))); /* from right */
          newsites|=
            ((sadx[row][previouscol]&cluster[row][previouscol])
                <<(WORDSIZE-1)); /*left */
          while (newsites) /* keep working on present word */
           {newsites|=
              (temporary&(currentword<<1)); /* grow to left*/
            newsites|=
              (((temporary&currentword)>>1)&(~LEFTBIT)); /* right */
            newsites&=(~currentword);
            currentword|=newsites;
           }
              /* are new sites added? ( single "=" intentional here) */
          if ((newsites=(currentword^cluster[row][col])))
             {clustergrowth=1;
              cluster[row][col]=currentword;
              rowloop-=(2*(rowloop>0)); /* back up */
              if (row<newboxbottom) 
               newboxbottom=row;
              if (row>newboxtop) 
               newboxtop=row;
              temp=col-(1&(newsites>>(WORDSIZE-1)));
              if (temp<newboxleft) newboxleft=temp;
              temp=col+(newsites&1);
              if (temp>newboxright) newboxright=temp;
             } /* end of if  new sites added */
         } /* end row loop */
       } /* end column loop */
if (clustergrowth)
 {if (newboxtop-newboxbottom<nrows-2)
   {boxbottom=newboxbottom+nrows-1;
    boxtop=newboxtop+nrows+2;
   }
  else
   {boxbottom=0;
    boxtop=nrows;
   }
  if (newboxright-newboxleft<hwords)
   {boxleft=newboxleft+hwords;
    boxright=newboxright+hwords+1;
   }
  else 
   {boxleft=0;
    boxright=hwords;
   }
   /* display cluster */
  XPutImage(display,window,gc,clusterimage,0,0,64,152+nrows,ncols,nrows); 
 }
 return;
} /* end of growcluster */

void check(row,col)
int row,col;
{ /* calculate sadx and sady when unchecked */
 /* demons are sad if they cannot flip a bond */
 long right,bottom,carry0,carry1;
 int bit;
 long index0,index1;
 right=field[row][col]^((field[row][col]<<1)
      |(1&(field[row][nextcol]>>(WORDSIZE-1))));
 bottom=field[row][col]^field[nextrow][col];
 if (boundary) /* correct for antiperiodic boundaries */
  {if (row==nrows-1)
     bottom^=(-1);
   if (col==topcol)
     right^=1;
  }  
 index0=(demonindex+2*(col+row*hwords)); /* start of xdemons */
 index1=(index0+1); /* start of ydemons */
 while (index0>=ndemons) index0-=ndemons;
 while (index1>=ndemons) index1-=ndemons;
 /* copy demons to newdemon */
 for (bit=0;bit<DEMONBITS;bit++)
  {newdemon[bit][index0]=demon[bit][index0];
   newdemon[bit][index1]=demon[bit][index1];
  }
 /* subtract 1 from newdemon */
 carry0=carry1=(-1);
 for (bit=0;bit<DEMONBITS;bit++)
  {newdemon[bit][index0]^=carry0;
   newdemon[bit][index1]^=carry1;
   carry0&=newdemon[bit][index0];
   carry1&=newdemon[bit][index1];
  }
 /* add two if neighbor antiparallel */
 for (bit=1;bit<DEMONBITS;bit++)
  {newdemon[bit][index0]^=right;
   newdemon[bit][index1]^=bottom;
   right&=(~newdemon[bit][index0]);
   bottom&=(~newdemon[bit][index1]);
  }
 carry0^=right;
 carry1^=bottom;
 /* make demons sad if overflow */
 sadx[row][col]=carry0;
 sady[row][col]=carry1;
 unchecked[row][col]=0;
 return;
}

void setup()
{int i,j,count;
/* initialize "bits" used for bitcounts */
 for (i=0;i<256;i++)
  {j=i;
   count=j&1;
   while (j) count+=((j>>=1)&1);
   bits[i]=count; /* the number of set bits in i */
  }
 /* make checkermask have alternate bits set */
 checkermask=2;
 for (i=0;i<32;i++) checkermask|=(checkermask<<2);
 return;
}

void repaint()
/* this fixes the window up whenever it is uncovered */
{int i;
 XSetPlaneMask(display,gc,AllPlanes); 
 drawbutton(quitbutton,0,0,BUTTONWIDTH,BUTTONHEIGHT,"quit",1);
 drawbutton(pausebutton,0,0,BUTTONWIDTH,BUTTONHEIGHT,"pause",1-2*paused);
 myalgo();
  /* draw the heater buttons */
 for (i=0;i<3;i++)
  drawbutton(heatwindow,0,i*HEATERHEIGHT/3,HEATERWIDTH,HEATERHEIGHT/3,
     heatertext[i],1-2*(i==(heater+2*cooler)));
  /* draw the boundary condition buttons */
 for (i=0;i<4;i++)
  drawbutton(boundwindow,0,i*BOUNDHEIGHT/4,BOUNDWIDTH,BOUNDHEIGHT/4,
     boundtext[i],1-2*(i==boundary));
  /* draw thermometer */
#define TTOP 128
#define THGT (nrows*2)
#define TBOTTOM (TTOP+THGT)
#define TLEFT 24
#define TWDTH 4
 XDrawLine(display,window,gc,TLEFT-3     ,TTOP-2   ,TLEFT+TWDTH+2,TTOP-2);
 XDrawLine(display,window,gc,TLEFT-3     ,TTOP-2   ,TLEFT-3      ,TBOTTOM+2);
 XDrawLine(display,window,gc,TLEFT-3     ,TBOTTOM+2,TLEFT+TWDTH+2,TBOTTOM+2);
 XDrawLine(display,window,gc,TLEFT+TWDTH+2,TBOTTOM+2,TLEFT+TWDTH+2,TTOP-2);
 for (i=TBOTTOM-1;i>TTOP;i-=(THGT/20))
  {XDrawLine(display,window,gc,TLEFT-7,i,TLEFT-4,i);
   XDrawLine(display,window,gc,TLEFT+TWDTH+3,i,TLEFT+TWDTH+6,i);
  }
 i=TBOTTOM-THGT*(-0.6+1.0/log(1.+sqrt(2.)));
 XDrawLine(display,window,gc,TLEFT-15,i,TLEFT+TWDTH+15,i); 
 XDrawString(display,window,gc,TLEFT+TWDTH+15,i+4,"T",1);
 XDrawString(display,window,gc,TLEFT+TWDTH+23,i+8,"c",1); 

  /* write various strings */
 sprintf(stringbuffer,"%d by %d lattice",ncols,nrows);
 XDrawString(display,window,gc,140,96,stringbuffer,strlen(stringbuffer));
 XDrawString(display,window,gc,20,96,"beta=",5);
 XDrawString(display,window,gc,64,114,"spins:",6);
 XDrawString(display,window,gc,64,142+nrows,"changes:",8);
 XDrawString(display,window,gcpen,TLEFT+ncols,112,"MJC",3);  
  
  /* draw border and redraw images */
 drawbutton(window,64-4,124-4,ncols+8,nrows+8,NULL,4);
 drawbutton(window,64-4,152+nrows-4,ncols+8,nrows+8,NULL,4);
 XPutImage(display,window,gc,spinimage,0,0,64,124,ncols,nrows); 
 XPutImage(display,window,gc,clusterimage,0,0,64,152+nrows,ncols,nrows); 

 fixthermometer(); 
 XSetPlaneMask(display,gc,black^white);
 return;
}

void refreshdemons()
/* refresh cluster demons at beta 
To generate a bit set with probability p:
write p as a binary fraction.  Compare a
random bit string with the digits of this 
fraction and find the first place at which
they are the same.  Output the bit  
that occupies that place.  This is done here
in parallel on WORDSIZE demons at a time. */
{long morebits,nrbit;
 double bfactor,prob,temp;
 long i;
 int bit,j,pbit[16];
 bfactor=exp(-beta);
 for (bit=0;bit<DEMONBITS;bit++)
    {bfactor*=bfactor; /* gives exp(-2 beta) for the first bit, 
                                exp(-4 beta) for the next */
     prob=bfactor/(1.+bfactor); /* prob that current demon bit is 1 */
     temp=prob;
       /* calculate the binary expression for p to 16 bit accuracy */
     for (j=0;j<16;j++) 
      {temp*=2; 
       pbit[j]=(temp>1.0);
       temp-=pbit[j];
      }
     for (i=0;i<ndemons;i++)
      {demon[bit][i]=0;
       morebits=(-1);
       for (j=0;j<16;j++)
        {nrbit=RND;  /* new set of random bits */
         if (pbit[j]) /* comparison bit is set */
          {demon[bit][i]|=(nrbit&morebits); /* if both are equal, set demon */
           morebits&=(~nrbit); /* finished with this demon */
          }
         else /* bit of p is zero */ 
          {morebits&=nrbit; /* finished with demons with zeros in nrbit */
          }
         if (0==morebits) break; /* all bits decided */
        }
      }
    }
  return;
}

void localupdate()
/* the local updating routine */
{long *up,*down; /* pointers to above and below rows */
 long temp0,temp1,reject,carry,spins,left,right,top,bottom;
 int row=0,col,rowloop;
 checkermask=(~checkermask);
 for (rowloop=0;rowloop<nrows;rowloop++)
  {row=(row+29); /* jump ahead 29 rows */
   while (row>=nrows) row-=nrows;
      /* get neighbors */
   up=field[previousrow];
   down=field[nextrow];
   leftshift( field[row],work0);
   rightshift(field[row],work1);
   for (col=0;col<hwords;col++)
    {spins=field[row][col];
        /* determine antiparallel neighbors */
     right=work0[col]^spins;    
     left=work1[col]^spins; 
     top=up[col]^spins;
     bottom=down[col]^spins; 
     if (0==row) 
     switch(boundary)
         {case 1:
           top^=(~0);
           break;
          case 2:
           top=(~0)^spins;
           break;
          case 3:
           top=spins;
           break;
         }  
     else if (nrows-1==row) 
        switch(boundary)
         {case 1:
           bottom^=(~0);
           break; 
          case 2:
           bottom=(~0)^spins;
           break;
          case 3:
           bottom=spins;
           break;
         }  
          /* add up top, bottom, and left antiparallel spins */
     temp0=top^bottom^left;
     temp1=(top&(bottom|left))|(bottom&left);
      /* add in right */
     carry=temp0&right; 
     temp0^=right;
     temp1^=carry;
     reject=carry&(~temp1);
      /* add  demon0 */
     carry=demon0&temp0;
     temp0^=demon0;
     temp1^=carry;
     reject^=(carry&(~temp1));
      /* add in demon1 */
     temp1^=demon1;
     reject^=(demon1&(~temp1));
      /*  subtract two */
     temp1=~temp1; 
     reject^=temp1;
      /* only change one color sites */
     reject|=checkermask;
      /* make changes */
     demon0=(demon0&reject)|(temp0&(~reject));
     demon1=(demon1&reject)|(temp1&(~reject));
     field[row][col]=(spins^(~reject)); 
     cluster[row][col]&=checkermask; 
     cluster[row][col]|=(~reject); 
     energycount+=bitcount(demon0&(~checkermask));
    }
           /* heat or cool */
   if (heater) 
       {if (drand48()<.04) demon0=RND;
       }
   else if (cooler) 
       {if (drand48()<.04) demon0=demon1=0;
       }
  }
 iteration=(iteration+1);
   /* monitor temperature every 10 "half" iterations */
 if (iteration==10) /* update beta and thermometer */
  {iteration=0;
   beta=.25*log((double) (-1.+10*volume/(2.*energycount)));
   fixthermometer(); 
   energycount=0;
  }
  /* copy lattice to window */
 XPutImage(display,window,gc,spinimage,0,0,64,124,ncols,nrows); 
 XPutImage(display,window,gc,clusterimage,0,0,64,152+nrows,ncols,nrows); 
 return;
}
 
void fixthermometer()
{int temp;
 sprintf(stringbuffer,"%.3f",beta);
 XDrawImageString(display,window,gcpen,64,96,
       stringbuffer,strlen(stringbuffer)); 
 temp=THGT*(-0.6+0.5/beta); 
 if (temp<0) temp=0;
 if  (temp>THGT) temp=THGT;
 XSetForeground(display,gcpen,darkcolor);
 XFillRectangle(display,window,gcpen,TLEFT,TBOTTOM-temp,TWDTH,temp);
 XFillRectangle(display,window,gcclear,TLEFT,TTOP,TWDTH,THGT-temp);
 return;
}

void rightshift(source,destination)
/* shifts a row of cells to the right by one bit */
long *source,*destination;
{int col;
 long carry=0;
 for (col=bottomcol;col!=(topcol+nshift);col+=nshift)
  {destination[col]=(~LEFTBIT)&(source[col]>>1);
   if (carry) destination[col]|=LEFTBIT;
   carry=source[col]&1;
  }
 if (carry) destination[bottomcol]|=LEFTBIT;
 switch(boundary)
   {case 1:
     destination[bottomcol]^=LEFTBIT;
     break;
    case 2:
     destination[bottomcol]|=LEFTBIT;
     break;
    case 3:
     destination[bottomcol]&=(~LEFTBIT);
     break;
   }
 return;
}

void leftshift(source,destination)
/* shifts a row of cells one bit to the left */
long *source,*destination;
{int col;
 long carry=0;
 for (col=topcol;col!=(bottomcol-nshift);col-=nshift)
  {destination[col]=(source[col]<<1);
   if (carry) destination[col]|=1;
   carry=source[col]>>(WORDSIZE-1);
  }
 if (carry) destination[topcol]|=1;
 switch(boundary)
   {case 1:
     destination[topcol]^=1;
     break;
   case 2:
     destination[topcol]|=1;
     break;
   case 3:
     destination[topcol]&=(~1);
     break;
   }
 return;
} 

void myalgo()
{int i;
 for (i=0;i<2;i++)
  drawbutton(algobutton,0,i*18,68,18,algotext[i],1-2*(algorithm==i));
 return;
}

void setalg()
/* take action if algorithm button hit*/
{int row,col;
 energycount=volumecount=iteration=0;
 if (1==algorithm) /* reset cluster stuff */
  {if (boundary>1)
     {setboundary(0);
     }
   refreshdemons(); 
   clustergrowth=0;
   for (col=0;col<hwords;col++)
   for (row=0;row<nrows;row++)
    {cluster[row][col]=0;
     unchecked[row][col]=1;
    }
  }
 return;
}

void setheater(value)
/* acts on heater buttons */
int value;
{int oldvalue;
 oldvalue=heater+2*cooler;
 value&=3;
 drawbutton(heatwindow,0,(HEATERHEIGHT*oldvalue)/3,
          HEATERWIDTH,HEATERHEIGHT/3,heatertext[oldvalue],1);
 heater=1&value;
 cooler=value>>1;
 drawbutton(heatwindow,0,(HEATERHEIGHT*value)/3,
          HEATERWIDTH,HEATERHEIGHT/3,heatertext[value],-1);
 return;
}

void setboundary(value)
/* take action if boundary buttons hit */
int value;
{drawbutton(boundwindow,0,boundary*BOUNDHEIGHT/4,BOUNDWIDTH,BOUNDHEIGHT/4,
     boundtext[boundary],1);
 /* don't allow fixed boundaries with cluster algorithm */
 if ((algorithm==1)&&(value>1))
  fprintf(stderr,"fixed boundaries not implemented for cluster algorithm\n");
 else
  boundary=value;
 drawbutton(boundwindow,0,boundary*BOUNDHEIGHT/4,BOUNDWIDTH,BOUNDHEIGHT/4,
     boundtext[boundary],-1);
 return;
}

void openwindow(argc,argv)
/* a lot of this is taken from the basicwin program in the
Xlib Programming Manual */
int argc;
char **argv;
{char *window_name="Microcanonical Ising Simulation";
 char *icon_name="Ising";
 Pixmap icon_pixmap;
 char *display_name=NULL;
 XColor xcolor,colorcell;
 Colormap cmap;
  /* icon */
#define icon_bitmap_width 16
#define icon_bitmap_height 16
 static unsigned char icon_bitmap_bits[] = {
   0x1f, 0xf8, 0x1f, 0x88, 0x1f, 0x88, 0x1f, 0x88, 0x1f, 0x88, 0x1f, 0xf8,
   0x1f, 0xf8, 0x1f, 0xf8, 0x1f, 0xf8, 0x1f, 0xf8, 0x1f, 0xf8, 0xff, 0xff,
   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

  /* open up the display */
 if ((display=XOpenDisplay(display_name))==NULL)
  {fprintf(stderr,"%s: cannot connect to X server %s\n",
    progname,XDisplayName(display_name));
   exit(-1);
  }
 screen=DefaultScreen(display);
 depth=DefaultDepth(display,screen);
 cmap=DefaultColormap(display,screen);
 /* depth=1; */  /* uncomment to test monochrome display */
 darkcolor=black=BlackPixel(display,screen);
 lightcolor=white=WhitePixel(display,screen);
 if (depth>1) /* color? This is not the right way to do it, but .... */
  {if (XAllocNamedColor(display,cmap,"firebrick",&colorcell,&xcolor))
              darkcolor=colorcell.pixel;
   if (XAllocNamedColor(display,cmap,"wheat",&colorcell,&xcolor))
              lightcolor=colorcell.pixel;
  }

  /* make the main window */
 windowwidth=ncols+75;
 windowheight=2*nrows+164;
 window=XCreateSimpleWindow(display,RootWindow(display,screen),
   0,0,windowwidth,windowheight,4,BlackPixel(display,screen),
   lightcolor);

  /* make the icon */
 icon_pixmap=XCreateBitmapFromData(display,window,
   icon_bitmap_bits,icon_bitmap_width,
   icon_bitmap_height);

 size_hints.flags=PPosition | PSize | PMinSize;
 size_hints.min_width=windowwidth;
 size_hints.min_height=200;
#ifdef X11R3
 size_hints.x=0;
 size_hints.y=0;
 size_hints.width=windowwidth;
 size_hints.height=windowheight;
 XSetStandardProperties(display,window,window_name,icon_name,
    icon_pixmap,argv,argc,&size_hints);
#else
 {XWMHints wm_hints;
  XClassHint class_hints;
  XTextProperty windowName, iconName;
  if (XStringListToTextProperty(&window_name,1,&windowName)==0)
   {fprintf(stderr,"%s: structure allocation for windowName failed.\n"
      ,progname);
    exit(-1);
   }
  if (XStringListToTextProperty(&icon_name,1,&iconName)==0)
   {fprintf(stderr,"%s: structure allocation for iconName failed.\n"
       ,progname);
    exit(-1);
   }
  wm_hints.initial_state=NormalState;
  wm_hints.input=True;
  wm_hints.icon_pixmap=icon_pixmap;
  wm_hints.flags=StateHint|IconPixmapHint|InputHint;
  class_hints.res_name=progname;
  class_hints.res_class="Basicwin";
  XSetWMProperties(display,window,&windowName,&iconName,
       argv,argc,&size_hints,&wm_hints,&class_hints);
 }
#endif

  /* pick the events to look for */
 event_mask=ExposureMask|ButtonPressMask|StructureNotifyMask;
 XSelectInput(display,window,event_mask);
  /* pick font: 9x15 is supposed to almost always be there */
 if ((font=XLoadQueryFont(display,"9x15"))==NULL)
  {fprintf(stderr,"%s: Cannot open 9x15 font\n",progname);
   exit(-1);
  }
 font_height=font->ascent+font->descent;
 font_width=font->max_bounds.width;
  /* make graphics contexts: 
      gc for black on white
      gcpen for various, defaults to darkcolor on lightcolor
      gclear does the reverse, clears thermometer top */ 
   
 gc=XCreateGC(display,window,0,NULL);
 XSetFont(display,gc,font->fid);
 XSetForeground(display,gc,black);
 XSetBackground(display,gc,white); 

 gcpen=XCreateGC(display,window,0,NULL); 
 XSetFont(display,gcpen,font->fid);
 XSetForeground(display,gcpen,darkcolor);
 XSetBackground(display,gcpen,lightcolor); 

 gcclear=XCreateGC(display,window,0,NULL); 
 XSetFont(display,gcclear,font->fid);
 XSetForeground(display,gcclear,lightcolor);
 XSetBackground(display,gcclear,darkcolor); 

/* show the window */
 XMapWindow(display,window);
 return;
}

void allocarray(array,xsize,ysize,datasize)
char ***array;
int xsize,ysize,datasize;
/* dynamically allocates a two dimensional (*array)[xize][ysize]
   with datasize bytes per element */
{int i;
 if ((*array)!=((char**) NULL))  /* free previous allocation */ 
   free(*array);
  /* allocate memory for array of xsize pointers plus data */
 *array=(char **) malloc(sizeof(char *)*xsize+datasize*xsize*ysize); 
 if (*array==(char**) NULL)
  {fprintf(stderr,"%s: memory allocation problems\n",progname);
   exit(-1);
  }
  /* initialize the pointers */
 for (i=0;i<xsize;i++)
  *((*array)+i)=((char *) (*array+xsize)+(i*ysize*datasize));
 return;
}

void makebuttons()
/* creates the buttons and allocates arrays; call after any resizing */
{int row,col;
 XDestroySubwindows(display,window);
 XSetForeground(display,gcpen,lightcolor);
 XFillRectangle(display,window,gcpen,0,0,windowwidth,windowheight);
 quitbutton  =makebutton(4,4,BUTTONWIDTH,BUTTONHEIGHT);
 pausebutton =makebutton(58,4,BUTTONWIDTH,BUTTONHEIGHT);
 boundwindow =makebutton(windowwidth-BOUNDWIDTH-4,4,BOUNDWIDTH,BOUNDHEIGHT);
 heatwindow  =makebutton(windowwidth-HEATERWIDTH-BOUNDWIDTH-12,4,
    HEATERWIDTH,HEATERHEIGHT);
 algobutton  =makebutton(18,36,68,36);

  /* reset various lattice dimensions */
 hwords=ncols/WORDSIZE;
 ncols=hwords*WORDSIZE; /* truncate ncols to a multiple of wordsize */
 if (0==(nrows%29)) nrows--; /* so demon jump of 29 rows works ok */
 volume=nrows*ncols;
 ndemons=2*nrows*hwords;

  /* allocate various arrays */
 if (NULL!=spinimage) 
  {(*spinimage).data=NULL;
   XDestroyImage(spinimage);
  }
 if (NULL!=clusterimage) 
  {(*clusterimage).data=NULL;
   XDestroyImage(clusterimage);
  }
 allocarray(&field,nrows,hwords,sizeof(long));
 allocarray(&cluster,nrows,hwords,sizeof(long));
 allocarray(&sadx,nrows,hwords,sizeof(long));
 allocarray(&sady,nrows,hwords,sizeof(long));
 allocarray(&unchecked,nrows,hwords,sizeof(long));
 allocarray(&demon,DEMONBITS,ndemons,sizeof(long));
 allocarray(&newdemon,DEMONBITS,ndemons,sizeof(long));
 if (NULL!=work0) free(work0);
 if (NULL!=work1) free(work1);
 work0=(long*) malloc(sizeof(long)*hwords);
 work1=(long*) malloc(sizeof(long)*hwords);
 if ((NULL==work0)||(NULL==work1))
  {fprintf(stderr,"%s: memory allocation problems\n",progname);
   exit(-1);
  }
 energycount=volumecount=iteration=0;

/* make image structures */
 spinimage=XCreateImage(display,(Visual *) &window,1,XYBitmap,0,
    (char*) *field,ncols,nrows,32,0);
 clusterimage=XCreateImage(display,(Visual *) &window,1,XYBitmap,0,
    (char*) *cluster,ncols,nrows,32,0);
 if ((NULL==spinimage)||(NULL==clusterimage))
  {fprintf(stderr,"%s: memory allocation problems\n",
    progname);
   exit(-1);
  }
 (*spinimage).bitmap_unit=WORDSIZE;
 (*clusterimage).bitmap_unit=WORDSIZE; 

/* test for byte order on client */
 **field=1;
 if ( * (char*) (*field))
  {(*spinimage).byte_order=LSBFirst;
   (*clusterimage).byte_order=LSBFirst;
     /* printf("byte_order=LSBFirst\n"); */
  }
 else
  {(*spinimage).byte_order=MSBFirst;
   (*clusterimage).byte_order=MSBFirst;
    /* printf("byte_order=MSBFirst\n"); */
  }
  /* find out in what order bits are put on screen */
 **field=0xff;
 if (XGetPixel(spinimage,0,0)) /* bits are backward */
    {nshift=(-1);
     topcol=0;
     bottomcol=hwords-1;
     (*spinimage).bitmap_bit_order=LSBFirst;
     (*clusterimage).bitmap_bit_order=LSBFirst;
     /* printf("bitmap_bit_order=LSBFirst\n"); */
    }
 else 
    {nshift=1;
     topcol=hwords-1;
     bottomcol=0;
     (*spinimage).bitmap_bit_order=MSBFirst;
     (*clusterimage).bitmap_bit_order=MSBFirst;
    }

   /* set initial state partially random and clear cluster array */
 for (col=0;col<hwords;col++)
 for (row=0;row<nrows;row++)
  {field[row][col]=~(RND*(drand48()>.77));
   cluster[row][col]=0;
   unchecked[row][col]=1;
  }
 beta=0.5*log(1.0+sqrt(2.0)); /* critical coupling */
 refreshdemons();
 return;
}

Window makebutton(xoffset,yoffset,xsize,ysize)
int xoffset,yoffset,xsize,ysize;
/* Puts button of specified dimensions on window.  Nothing is drawn. */
{Window buttonwindow;
 long event_mask;
 buttonwindow=XCreateSimpleWindow(display,window,xoffset,yoffset,
        xsize,ysize,0,black,lightcolor);
 event_mask=ButtonPressMask|ExposureMask; /* look for mouse-button presses */
 XSelectInput(display,buttonwindow,event_mask);
 XMapWindow(display,buttonwindow);
 return buttonwindow;
}

void drawbutton(buttonwindow,xoffset,yoffset,xsize,ysize,text,state)
Window buttonwindow;
int xoffset,yoffset,xsize,ysize,state;
char * text;
/* Draw a button in buttonwindow of specified dimensions with text
   centered.  Color of button determined by sign of "state." 
   size of border determined by magnitude. */
{int textlength,i,j;
 int cdark,clight,cup,cdown;
 int cleft,cright,cbutton,ctext;
 cup=lightcolor;
 cdown=darkcolor;
 cdark=black;
 clight=white;
 if (state<0)
  {cbutton=cdown;
   ctext=clight;
   cleft=cdark;
   cright=clight;
  } 
 else
  {cbutton=cup;
   ctext=cdark;
   cleft=clight;
   cright=cdark;
  }
 if (1==depth) clight=cleft=cright=black;
 j=abs(state);
 XSetForeground(display,gcpen,cbutton);  
 XFillRectangle(display,buttonwindow,gcpen,xoffset+j,yoffset+j,
        xsize-2*j,ysize-2*j);
 XSetForeground(display,gcpen,cleft);
 XFillRectangle(display,buttonwindow,gcpen,xoffset,yoffset,xsize,j);
 XFillRectangle(display,buttonwindow,gcpen,xoffset,yoffset,j,ysize);
 XSetForeground(display,gcpen,cright);
 for (i=0;i<j;i++) 
  {XDrawLine(display,buttonwindow,gcpen,
     xoffset+i,yoffset+ysize-i-1,xoffset+xsize-i-1,yoffset+ysize-i-1);
   XDrawLine(display,buttonwindow,gcpen,
     xoffset+xsize-i-1,yoffset+i,xoffset+xsize-i-1,yoffset+ysize-i-1);
  }
 if (NULL!=text)
   {textlength=strlen(text);
    XSetForeground(display,gcpen,ctext);
    XDrawString(display,buttonwindow,gcpen,
          xoffset+(xsize-textlength*font_width)/2,
          yoffset+(ysize+font_height)/2-2,
          text,textlength);
   }
 XSetForeground(display,gcpen,darkcolor);  
 return;
}

